import json

from wordfreak.parser.DocxParser import DocxParser
from wordfreak.parser.PdfParser import PdfParser
from wordfreak.parser.TxtParser import TxtParser


def extractWordFrequencies(inputFilePath: str, outputFilePath: str) -> None:
    """
    Takes a file path and saves all word frequencies to the given JSON file.

    inputFilePath: Path to file to extract work frequencies from.
    outputFilePath: Path to file to save word frequencies to (must be .json file).
    """
    DOCX_FILE_EXTENSION = ".docx"
    JSON_FILE_EXTENSION = ".json"
    PDF_FILE_EXTENSION = ".pdf"
    TXT_FILE_EXTENSION = ".txt"
    
    if not outputFilePath.lower().endswith(JSON_FILE_EXTENSION):
        raise ValueError(f"Output file must be a .json file.")

    if inputFilePath.lower().endswith(TXT_FILE_EXTENSION):
        wordFrequencies = TxtParser.getWordFrequency(inputFilePath)
    elif inputFilePath.lower().endswith(PDF_FILE_EXTENSION):
        wordFrequencies = PdfParser.getWordFrequency(inputFilePath)
    elif inputFilePath.lower().endswith(DOCX_FILE_EXTENSION):
        wordFrequencies = DocxParser.getWordFrequency(inputFilePath)
    else:
        raise ValueError(f"Filetype not supported for parsing (tried to parse file at '{inputFilePath}').")

    # sort word frequencies by number of occurrences
    orderedWordFreq = dict(sorted(wordFrequencies.items(), reverse=True, key=lambda item: item[1]))

    # save to JSON file
    with open(outputFilePath, "w+") as file:
        json.dump(orderedWordFreq, file)



import string
from abc import abstractmethod, ABC


class Parser(ABC):

    @classmethod
    @abstractmethod
    def getWordFrequency(cls, pathToFile: str) -> dict[str, int]:
        ...

    @classmethod
    def _getWordFrequencyFromLines(cls, lines: list[str]) -> dict[str, int]:
        wordFrequency: dict[str, int] = dict()
        for line in lines:
            line = line.strip()
            words = line.split()
            for word in words:
                word = word.translate(str.maketrans('', '', string.punctuation)).lower()
                # do not add empty strings
                if word != "":
                    if word in wordFrequency:
                        wordFrequency[word] += 1
                    else:
                        wordFrequency[word] = 1
        return wordFrequency



import setuptools

with open("LICENSE") as f:
    license = f.read()

setuptools.setup(
    name="wordfreak",
    version="0.0.8",
    author="Joey Greco",
    author_email="joeyagreco@gmail.com",
    description="Word Freak is a Python library that extracts word frequencies from files.",
    license=license,
    packages=setuptools.find_packages(exclude=("test", "docs")),
    install_requires=["PyPDF2",
                      "docx2txt",
                      "setuptools"]
)
